@@ -21,7 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <glib.h>
 #include "gdk.h"		/* gdk_event_send_client_message() */
 #include "gdkdisplay.h"
@@ -258,6 +258,8 @@ gdk_display_put_event (GdkDisplay     *display,
   g_return_if_fail (event != NULL);
 
   _gdk_event_queue_append (display, gdk_event_copy (event));
+  /* If the main loop is blocking in a different thread, wake it up */
+  g_main_context_wakeup (NULL); 
 }
 
 /**
@@ -474,13 +476,15 @@ gdk_display_get_pointer (GdkDisplay      *display,
 /**
  * gdk_display_get_window_at_pointer:
  * @display: a #GdkDisplay
- * @win_x: return location for origin of the window under the pointer
- * @win_y: return location for origin of the window under the pointer
+ * @win_x: return location for x coordinate of the pointer location relative 
+ *    to the window origin, or %NULL
+ * @win_y: return location for y coordinate of the pointer location relative
+ &    to the window origin, or %NULL
  * 
  * Obtains the window underneath the mouse pointer, returning the location
- * of that window in @win_x, @win_y for @screen. Returns %NULL if the window 
- * under the mouse pointer is not known to GDK (for example, belongs to
- * another application).
+ * of the pointer in that window in @win_x, @win_y for @screen. Returns %NULL 
+ * if the window under the mouse pointer is not known to GDK (for example, 
+ * belongs to another application).
  * 
  * Returns: the window under the mouse pointer, or %NULL
  *
