@@ -440,6 +440,10 @@ gtk_tooltip_trigger_tooltip_query (GdkDisplay *display)
   event.motion.y = y;
   event.motion.is_hint = FALSE;
 
+  gdk_window_get_origin (window, &x, &y);
+  event.motion.x_root = event.motion.x + x;
+  event.motion.y_root = event.motion.y + y;
+
   _gtk_tooltip_handle_event (&event);
 }
 
@@ -518,6 +522,9 @@ child_location_foreach (GtkWidget *child,
   if (!GTK_WIDGET_DRAWABLE (child))
     return;
 
+  x = 0;
+  y = 0;
+
   /* (child_loc->x, child_loc->y) are relative to
    * child_loc->container's allocation.
    */
@@ -911,6 +918,7 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
 
   if (tooltip->keyboard_mode_enabled)
     {
+      x = y = -1;
       pointer_widget = tooltip_widget = tooltip->keyboard_widget;
     }
   else
@@ -933,8 +941,6 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
 
   g_object_get (tooltip_widget, "has-tooltip", &has_tooltip, NULL);
 
-  g_assert (tooltip != NULL);
-
   return_value = gtk_tooltip_run_requery (&tooltip_widget, tooltip, &x, &y);
   if (!return_value)
     return;
@@ -1036,11 +1042,17 @@ tooltip_popup_timeout (gpointer data)
   GtkTooltip *tooltip;
 
   display = GDK_DISPLAY_OBJECT (data);
+  tooltip = g_object_get_data (G_OBJECT (display),
+			       "gdk-display-current-tooltip");
+
+  /* This usually does not happen.  However, it does occur in language
+   * bindings were reference counting of objects behaves differently.
+   */
+  if (!tooltip)
+    return FALSE;
 
   gtk_tooltip_show_tooltip (display);
 
-  tooltip = g_object_get_data (G_OBJECT (display),
-			       "gdk-display-current-tooltip");
   tooltip->timeout_id = 0;
 
   return FALSE;
@@ -1056,7 +1068,7 @@ gtk_tooltip_start_delay (GdkDisplay *display)
   tooltip = g_object_get_data (G_OBJECT (display),
 			       "gdk-display-current-tooltip");
 
-  if (tooltip && GTK_TOOLTIP_VISIBLE (tooltip))
+  if (!tooltip || GTK_TOOLTIP_VISIBLE (tooltip))
     return;
 
   if (tooltip->timeout_id)
